import javafx.application.Application;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;
import javafx.stage.Stage;
import javafx.scene.text.Text;
//********************************************
//  Demonstrates handling of mouse events
//  and computing distances and angles
//********************************************

public class Lines extends Application
{
    private Line line;
    private Group root, arrow;
    private Text distText, angleText;

    public void start(Stage primaryStage)
    {
        distText = new Text(10,25,"Distance: ");
        angleText = new Text(10,10,"Angle: ");
        line = new Line();
        line.setStroke(Color.GREEN);
        line.setStrokeWidth(3);

        arrow = new Group(new Line(50,100,100,100),
                          new Line(80,80,100,100),
                          new Line(80,120,100,100));

        root = new Group(distText, angleText, line, arrow);

        Scene scene = new Scene(root, 500, 300, Color.WHITE);

        scene.setOnMousePressed(this::processMousePress);
        scene.setOnMouseDragged(this::processMouseDrag);
        scene.setOnMouseMoved(this::processMouseMoved);

        primaryStage.setTitle("Lines");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    public void processMousePress(MouseEvent event)
    {
        line.setStartX(event.getX());
        line.setStartY(event.getY());
        line.setEndX(event.getX());
        line.setEndX(event.getY());
    }

    //--------------------------------------------------------------------
    //  Updates the end point of the current line as the mouse is
    //  dragged, creating the rubber band effect.
    //--------------------------------------------------------------------
    public void processMouseDrag(MouseEvent event)
    {
        line.setEndX(event.getX());
        line.setEndY(event.getY());
        double x1 = line.getStartX();
        double y1 = line.getStartY();
        double x2 = line.getEndX();
        double y2 = line.getEndY();
        double alpha = Math.atan((y2-y1)/(x2-x1));
        if (x1>x2) alpha = alpha + Math.PI;
        if (x1<x2 && y1>y2) alpha = alpha + 2*Math.PI;
        angleText.setText("Angle: "+alpha);
        arrow.setRotate(alpha*180/Math.PI);
    }

    public void processMouseMoved(MouseEvent event)
    {
        double x1 = line.getStartX();
        double y1 = line.getStartY();
        double x2 = line.getEndX();
        double y2 = line.getEndY();
        double x3 = event.getX();
        double y3 = event.getY();
        distText.setText("Distance: "+ (distance(x1,y1,x3,y3)+
                distance(x2,y2,x3,y3)-distance(x1,y1,x2,y2)));
    }

    private double distance(double x1,double y1,double x2,double y2)
    {
        return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
    }
}
